import asyncio
import datetime
import logging
import os
import random  # nosec
from asyncio import subprocess

from pysolar import solar

logger = logging.getLogger(__package__)


async def update_color(latitude, longitude, leds, clouds, sound, interval=5):
    warm, cold = leds
    max_cloud_duration = 30
    cloudy = 0
    player = None
    if sound is not None and not os.path.isfile(sound):
        raise ValueError("File does not exist: %s" % sound)
    while True:
        now = datetime.datetime.now(datetime.timezone.utc)
        sun_altitude_deg = solar.get_altitude(latitude, longitude, now)
        logger.info("Sun altitude: %.1fº", sun_altitude_deg)

        if sound and player is None and sun_altitude_deg > -8:
            player = await subprocess.create_subprocess_exec(
                "ffplay", "-loop", "0", "-loglevel", "24", sound
            )

        if sun_altitude_deg > 16:
            if cloudy:
                cloudy -= interval
                asyncio.Task(warm.fade(0))
            else:
                cloudy = int(random.random() < clouds) * random.randrange(  # nosec
                    0, max_cloud_duration, interval
                )
                asyncio.Task(warm.fade(1))
            cold.value = 1
        elif sun_altitude_deg > 8:
            warm.value = 1
            asyncio.Task(cold.fade((sun_altitude_deg + 8) / 24))
        elif sun_altitude_deg > -8:
            cold.value = 0
            asyncio.Task(warm.fade(max((sun_altitude_deg + 8) / 16, 0.001)))
        else:
            cold.value = 0
            warm.value = 0
            if player is not None:
                player.terminate()
                player = None
        await asyncio.sleep(interval)



import asyncio
import logging
import sys

import click

from . import led
from .utils import update_color

LOG_FORMAT = "%(asctime)s %(levelname)-8s %(message)s"

logger = logging.getLogger(__package__)


@click.command()
@click.argument("latitude", type=float)
@click.argument("longitude", type=float)
@click.option(
    "--clouds",
    "-c",
    default=0.1,
    help="Cache of clouds between. Between 0 and 1, default 0.1.",
)
@click.option(
    "--host", "-h", default="localhost", help="PiGPIO host, default: localhost"
)
@click.option("--warm", default=12, help="Warm LED GPIO pin, default: 12")
@click.option("--cold", default=13, help="Warm LED GPIO pin, default: 13")
@click.option("--sound", help="Path to sound that will be looped during the day.")
@click.option(
    "--verbosity", "-v", default=0, count=True, help="Increase output verbosity."
)
def main(latitude, longitude, clouds, host, warm, cold, sound, verbosity):
    setup_logging(verbosity)
    leds = led.init(host, warm, cold)
    asyncio.run(update_color(latitude, longitude, leds, clouds, sound))


def get_logging_level(verbosity):
    level = logging.WARNING
    level -= verbosity * 10
    if level < logging.DEBUG:
        level = logging.DEBUG
    return level


def setup_logging(verbosity):
    hdlr = logging.StreamHandler(sys.stdout)
    hdlr.setLevel(get_logging_level(verbosity))
    hdlr.setFormatter(logging.Formatter(LOG_FORMAT))
    logger.addHandler(hdlr)
    logger.setLevel(get_logging_level(verbosity))


if __name__ == "__main__":
    sys.exit(main())
